@@ -2,37 +2,56 @@
 # unique units in the synthesised data that replicates unique real units
 # (+number +percent of all observations)
 
-replicated.uniques <- function(object,data){
-  # extract uniques from the original data
-  uReal <- data[!(duplicated(data) | duplicated(data,fromLast=TRUE)),]
- 
-  if (object$m==1){
-    Syn    <- object$syn
-    rm.Syn <- rep(FALSE,nrow(Syn))
-    i.unique.Syn <- which(!(duplicated(Syn) | duplicated(Syn,fromLast=TRUE)))  
-    uSyn <- Syn[i.unique.Syn,]
-    uAll <- rbind.data.frame(uReal,uSyn)
-    dup.of.unique <- duplicated(uAll)[(nrow(uReal)+1):nrow(uAll)]
-    rm.Syn[i.unique.Syn] <- dup.of.unique
-    no.duplicates <- sum(dup.of.unique)
+replicated.uniques <- function(object, data, exclude = NULL){
+  # check names of vars to be excluded
+  if (!is.null(exclude)) {
+    exclude.cols <- match(exclude, colnames(data))
+    if (any(is.na(exclude.cols))) stop("Unrecognized variable(s) in exclude parameter for uniques: ",
+      paste(exclude[is.na(exclude.cols)],collapse=", "), call. = FALSE)
   }
   
-  if (object$m>1){
-  rm.Syn <- matrix(FALSE,nrow=nrow(object$syn[[1]]),ncol=object$m)
-    for (i in 1:object$m){
-      Syn  <- object$syn[[i]]
-      i.unique.Syn <- which(!(duplicated(Syn) | duplicated(Syn,fromLast=TRUE)))  
+  # exclude vars from the original data
+  if (!is.null(exclude)) data <- data[,-exclude.cols, drop = FALSE] else data <- data
+  # extract uniques from the original data
+  uReal <- data[!(duplicated(data) | duplicated(data,fromLast=TRUE)),]
+  no.uniques <- nrow(uReal)
+
+  if (is.null(no.uniques) || no.uniques == 0) {
+    no.uniques <- 0
+    no.duplicates <- per.duplicates <- rep(0,object$m)
+    if (object$m == 1) rm.Syn <- rep(FALSE,nrow(object$syn))
+    if (object$m > 1) rm.Syn <- matrix(FALSE,nrow=nrow(object$syn[[1]]),ncol=object$m)
+
+  } else {
+
+    if (object$m == 1){
+      if (!is.null(exclude)) Syn <- object$syn[,-exclude.cols, drop = FALSE] else Syn <- object$syn
+      rm.Syn <- rep(FALSE,nrow(Syn))
+      i.unique.Syn <- which(!(duplicated(Syn) | duplicated(Syn,fromLast=TRUE)))
       uSyn <- Syn[i.unique.Syn,]
       uAll <- rbind.data.frame(uReal,uSyn)
       dup.of.unique <- duplicated(uAll)[(nrow(uReal)+1):nrow(uAll)]
-      rm.Syn[i.unique.Syn,i] <- dup.of.unique
-    } 
-    no.duplicates <- colSums(rm.Syn)
+      rm.Syn[i.unique.Syn] <- dup.of.unique
+      no.duplicates <- sum(dup.of.unique)
+    }
+
+    if (object$m > 1){
+    rm.Syn <- matrix(FALSE,nrow=nrow(object$syn[[1]]),ncol=object$m)
+      for (i in 1:object$m){
+        if (!is.null(exclude)) Syn <- object$syn[[i]][,-exclude.cols,drop = FALSE] else Syn <- object$syn[[i]]
+        i.unique.Syn <- which(!(duplicated(Syn) | duplicated(Syn,fromLast=TRUE)))
+        uSyn <- Syn[i.unique.Syn,]
+        uAll <- rbind.data.frame(uReal,uSyn)
+        dup.of.unique <- duplicated(uAll)[(nrow(uReal)+1):nrow(uAll)]
+        rm.Syn[i.unique.Syn,i] <- dup.of.unique
+      }
+      no.duplicates <- colSums(rm.Syn)
+    }
+    per.duplicates <- no.duplicates/nrow(data)*100
   }
 
-  per.duplicates <- no.duplicates/nrow(data)*100
-  return(list(replications=rm.Syn,no.replications=no.duplicates,
-              per.replications=per.duplicates))
+  return(list(replications = rm.Syn, no.uniques = no.uniques,
+    no.replications = no.duplicates, per.replications = per.duplicates))
 }
 
 
@@ -40,8 +59,9 @@ replicated.uniques <- function(object,data){
 # sdc - statistical disclosure control:
 # labeling, removing unique replicates of unique real individuals
 
-sdc <- function(object,data,label=NULL,rm.replicated.uniques=FALSE,
- recode.vars=NULL,bottom.top.coding=NULL,recode.exclude=NULL){
+sdc <- function(object, data, label = NULL, rm.replicated.uniques = FALSE, 
+ uniques.exclude = NULL, recode.vars = NULL, bottom.top.coding = NULL,
+ recode.exclude = NULL){
 
  if (!is.null(recode.vars)){
    if (!is.null(bottom.top.coding) && !is.list(bottom.top.coding)) 
@@ -67,15 +87,14 @@ sdc <- function(object,data,label=NULL,rm.replicated.uniques=FALSE,
        object$syn[,i] <- recoded$x
        cat("\n",recode.vars[j],": no. of bottom-coded values - ",
          recoded$no.recoded.bottom,", no. of top-coded values - ",
-         recoded$no.recoded.top,sep="")
+         recoded$no.recoded.top, sep="")
      }
    cat("\n")
    }
    if (rm.replicated.uniques) {
-     du <- replicated.uniques(object,data) 
+     du <- replicated.uniques(object, data, exclude = uniques.exclude) 
      object$syn <- object$syn[!du$replications,]
-     if (du$no.replications>0) cat("\nNumber of unique units in the synthesised data that replicates unique real individuals:\n",
-       du$no.replications,"\nThey have been removed from the synthetic data set.\n\n")
+     cat("no. of replicated uniques: ", du$no.replications, "\n", sep="")
    }
    if (!is.null(label)) object$syn <- cbind.data.frame(flag=label, object$syn)
  }
@@ -90,20 +109,20 @@ sdc <- function(object,data,label=NULL,rm.replicated.uniques=FALSE,
          recoded <- bottom.top.recoding(object$syn[[k]][,i],bottom.top.coding[[j]][1],
            bottom.top.coding[[j]][2],recode.exclude[[j]])
          object$syn[[k]][,i] <- recoded$x
-         cat("\n",recode.vars[j],": no. of bottom-coded values - ",
-         recoded$no.recoded.bottom,", no. of top-coded values - ",
-         recoded$no.recoded.top,sep="")
+         cat("\n",recode.vars[j], ": no. of bottom-coded values - ",
+         recoded$no.recoded.bottom, ", no. of top-coded values - ",
+         recoded$no.recoded.top, sep="")
        }
      }
    cat("\n")
    }
    if (rm.replicated.uniques) {
-     du <- replicated.uniques(object,data) 
+     du <- replicated.uniques(object, data, exclude = uniques.exclude) 
      for (i in 1:object$m){ 
        object$syn[[i]] <- object$syn[[i]][!du$replications[,i],]
      }
-   if (any(du$no.replications>0)) cat("\nNo. of unique units in the synthesised data that replicates unique real individuals:\n",
-     du$no.replications,"\nThey have been removed from the synthetic data set(s).\n\n")
+   cat("no. of replicated uniques: ", 
+     paste0(du$no.replications, collapse=", "),"\n", sep="")
    }
    if (!is.null(label)) object$syn <- mapply(cbind.data.frame, flag=label,
      object$syn, SIMPLIFY=FALSE, USE.NAMES=FALSE)
@@ -213,136 +232,96 @@ read.obs <- function(file, convert.factors = TRUE, lab.factors = FALSE,
 
 write.syn <- function(object, filename,
   filetype = c("SPSS","Stata","SAS","csv","tab","rda","RData","txt"),
-  convert.factors = "numeric", data.labels = NULL, 
+  convert.factors = "numeric", data.labels = NULL,
   save.complete = TRUE, extended.info = TRUE, ...){
 
 # pos <- regexpr("\\.([[:alnum:]]+)$", file)
 # ext <- ifelse(pos > -1L, substring(file, pos + 1L), "")
-# without.ext <- strsplit(file,"\\.")[[1]][1]      #! will include path 
+# without.ext <- strsplit(file,"\\.")[[1]][1]      #! will include path
 
- if (is.null(data.labels)) data.labels <- list(var.lab = object$var.lab, 
+ m <- object$m
+ if (m == 1) object$syn <- list(object$syn)
+ filetype <- match.arg(filetype)
+ if (is.null(data.labels)) data.labels <- list(var.lab = object$var.lab,
                                                val.lab = object$val.lab)
- 
- #sink("synthesis_info.txt")
- call <- match.call()
- infofile <- paste("synthesis_info_",call$object,".txt",sep="")
- sink(infofile)
 
- cat("Date saved:",format(Sys.time(),"%d %b %Y, %H:%M"),"\n")
- cat("Data frame with original data:",object$call$data,"\n")
- cat("Number of synthetic data sets:",object$m,"\n")
- cat("Output file(s): ")
-
- if (object$m==1){
-
-   if (filetype=="SPSS") {
+ if (filetype=="SPSS"){
+   if (m==1) {
      f1 <- paste0(filename,".sps"); f2 <- paste0(filename,".txt")
-     #write.foreign(object$syn, codefile=f1, datafile=f2, package=filetype, ...)
-     write.syn.SPSS(object$syn, codefile=f1, datafile=f2, 
-       varnames = names(object$syn), data.labels = data.labels, ...)  
-   
-   } else if (filetype=="Stata") {
-     f1 <- paste0(filename,".dta")
-     write.dta(object$syn, file=f1, convert.factors=convert.factors, ...)
-     #!### check why default (better) convert.factors="labels" cuts the names
-
-   } else if (filetype=="SAS") {
-     f1 <- paste0(filename,".sas"); f2 <- paste0(filename,".txt")
-     write.foreign(object$syn, codefile=f1, datafile=f2, package=filetype, ...)
-
-   } else if (filetype=="rda" | filetype=="RData") {
-     syn <- object$syn
-     f1  <- paste0(filename,".rda")
-     save(syn,file=f1,...)
-
-   } else if (filetype=="csv") {
-     f1 <- paste0(filename,".csv")
-     write.csv(object$syn,file=f1, row.names=FALSE,...)
-
-   } else if (filetype=="txt") {
-     f1 <- paste0(filename,".txt")
-     write.table(object$syn,file=f1, row.names=FALSE,...)
-
-   } else if (filetype=="tab") {
-     f1 <- paste0(filename,".tab")
-     write.table(object$syn,file=f1, row.names=FALSE, sep="\t", ...)
-   }
-
-   if (filetype=="SPSS" | filetype=="SAS"){
-     cat(paste(paste0("(",filetype,")"), f1, f2, collapse=" "))
    } else {
-     cat(paste0("(",filetype,") ",f1))
+     f1 <- paste0(filename,"_",1:m,".sps"); f2 <- paste0(filename,"_",1:m,".txt")
    }
-
- }
-
- if (object$m>1){
- cat(paste0("(",filetype,")"))
-
-   if (filetype=="SPSS") {
-     for (i in 1:object$m){
-       f1 <- paste0(filename,i,".sps"); f2 <- paste0(filename,i,".txt")
-       #write.foreign(object$syn[[i]], codefile=f1, datafile=f2, package=filetype, ...)
-       write.syn.SPSS(object$syn[[i]], codefile=f1, datafile=f2, 
-         varnames = names(object$syn[[i]]), data.labels = data.labels, ...)
-       cat("\n",f1,f2,collapse=" ")
-     }
-   } else if (filetype=="Stata") {
-     for (i in 1:object$m){
-       f1 <- paste0(filename,i,".dta")
-       write.dta(object$syn[[i]], file=f1, convert.factors=convert.factors, ...)
+   for (i in 1:m) {
+     #write.foreign(object$syn[[i]], codefile=f1, datafile=f2, package=filetype, ...)
+     write.syn.SPSS(object$syn[[i]], codefile = f1[i], datafile = f2[i],
+       varnames = names(object$syn[[i]]), data.labels = data.labels, ...)
+   }
+ } else if (filetype == "Stata"){
+   if (m==1) f1 <- paste0(filename,".dta") else f1 <- paste0(filename,"_",1:m,".dta")
+   for (i in 1:m){
+     write.dta(object$syn[[i]], file = f1[i], convert.factors = convert.factors, ...)
        #!### check why default convert.factors="labels" cuts the names
-       cat("\n",f1)
-     }
-   } else if (filetype=="SAS") {
-     for (i in 1:object$m){
-       f1 <- paste0(filename,i,".sas"); f2 <- paste0(filename,i,".txt")
-       write.foreign(object$syn[[i]], codefile=f1, datafile=f2, package=filetype, ...)
-       cat("\n",f1,f2,collapse=" ")
-     }
-   } else if (filetype=="rda" | filetype=="RData") {
-     for (i in 1:object$m){
-       syn <- object$syn[[i]]
-       f1  <- paste0(filename,i,".rda")
-       save(syn,file=f1,...)
-       cat("\n",f1)
-     }
-   } else if (filetype=="csv") {
-     for (i in 1:object$m){
-       f1 <- paste0(filename,i,".csv")
-       write.csv(object$syn[[i]],file=f1, row.names=FALSE,...)
-       cat("\n",f1)
-     }
-   } else if (filetype=="txt") {
-     for (i in 1:object$m){
-       f1 <- paste0(filename,i,".txt")
-       write.table(object$syn[[i]],file=f1, row.names=FALSE,...)
-       cat("\n",f1)
-     }
-   } else if (filetype=="tab") {
-     for (i in 1:object$m){
-       f1 <- paste0(filename,i,".tab")
-       write.table(object$syn[[i]],file=f1, row.names=FALSE, sep="\t", ...)
-       cat("\n",f1)
-     }
    }
-
+ } else if (filetype == "SAS"){
+   if (m==1) {
+     f1 <- paste0(filename,".sas"); f2 <- paste0(filename,".txt")
+   } else {
+     f1 <- paste0(filename,"_",1:m,".sas"); f2 <- paste0(filename,"_",1:m,".txt")
+   }
+   for (i in 1:m){
+     write.foreign(object$syn[[i]], codefile = f1[i], datafile = f2[i], package = filetype, ...)
+   }
+ } else if (filetype == "rda" | filetype == "RData"){
+   if (m==1) f1 <- paste(filename,filetype,sep=".") else f1 <- paste0(filename,"_",1:m,".",filetype)
+   for (i in 1:m){
+     syn <- object$syn[[i]]
+     save(syn, file = f1[i],...)
+   }
+ } else if (filetype == "csv"){
+   if (m==1) f1 <- paste0(filename,".csv") else f1 <- paste0(filename,"_",1:m,".csv")
+   for (i in 1:m){
+     write.csv(object$syn[[i]], file = f1[i], row.names = FALSE, ...)
+   }
+ } else if (filetype == "txt"){
+   if (m==1) f1 <- paste0(filename,".txt") else f1 <- paste0(filename,"_",1:m,".txt")
+   for (i in 1:m){
+     write.table(object$syn[[i]], file = f1[i], row.names = FALSE, ...)
+   }
+ } else if (filetype == "tab"){
+   if (m==1) f1 <- paste0(filename,".tab") else f1 <- paste0(filename,"_",1:m,".tab")
+   for (i in 1:m){
+     write.table(object$syn[[i]],file=f1[i], row.names=FALSE, sep="\t", ...)
+   }
  }
 
- if (save.complete) {
-   save(object,file="synobject.rda")
-   cat("\nAdditional file: synobject.rda")
+ call <- match.call()
+ infofile <- paste("synthesis_info_",filename,".txt",sep="")
+  #---
+ sink(infofile)
+ cat("Date saved:",format(Sys.time(), "%d %b %Y, %H:%M"), "\n")
+ cat("Data frame with original data:", deparse(object$call$data), "\n")
+ cat("Number of synthetic data sets:", m, "\n")
+ cat("Output file(s):")
+ if (filetype=="SPSS" | filetype=="SAS"){
+   cat(paste0("(",filetype,") "))
+   cat(paste(f1, f2, collapse=" "))
+ } else {
+   cat(paste0("(",filetype,") ",f1))
+ }
+ if (save.complete) {                            
+   addname <- paste0("synobject_",filename,".RData")
+   save(object,file=addname)
+   cat("\nAdditional file: ", addname, sep="")
  }
-
  if (extended.info) {
    cat("\nMethods used:\n")
    print(object$method)
    cat("Seed used:",object$seed,"\n")
  }
-
  sink()
+ #---
 
- cat("Synthetic data exported as ",filetype," file.",sep="")
+ cat("Synthetic data exported as ",filetype," file(s).",sep="")
  cat("\nInformation on synthetic data written to\n ",
      paste(getwd(),"/",infofile,sep=""),"\n")
 
