@@ -2,7 +2,7 @@
 
 sampler.syn <- function(p, data, m, syn, visit.sequence,
                         rules, rvalues, event, proper,
-                        print.flag, k, pred.not.syn, ...){
+                        print.flag, k, pred.not.syn, models, ...){
 # The sampler controls the generation of conditional distributions
 # This function is called by syn
 
@@ -23,12 +23,13 @@ sampler.syn <- function(p, data, m, syn, visit.sequence,
   #method.args  <- sapply(method.args0,intersect,dots.names)
   #method.args  <- method.args[sapply(method.args,length)>0]
   #---
-  # browser()
+
+
   #--- Assign optional parameters (...) to appropriate synthesising function   
   dots  <- as.list(substitute(list(...)))[-1L]         
   meth.with.opt <- paste(c("cart","cartbboot","ctree","survctree","polyreg","polr","rf","bag"), collapse="\\.|")
   meth.check <- grep(meth.with.opt,names(dots),value=TRUE)
-  args.err   <- !(names(dots) %in% meth.check)
+  args.err <- !(names(dots) %in% meth.check)
   if (any(args.err)) stop("Unknown optional parameter(s): ", 
     paste(names(dots)[args.err],collapse=", "),
     "\nNote that they have to be method specific, e.g. 'ctree.minbucket' and NOT 'minbucket'\n", 
@@ -47,9 +48,15 @@ sampler.syn <- function(p, data, m, syn, visit.sequence,
     } 
   } 
   #---
+  
+  fits <- NULL 
     
   if (m > 0){
 	  if (print.flag) cat("syn  variables")
+    
+    if (models) fits <- rep(list(setNames(vector("list", length(p$method)),
+      names(p$method))), m) 
+
     for (i in 1:m){  # begin i loop : repeated synthesising loop
       if (print.flag) cat("\n",i,"   ",sep="")
 ## augment the data with the actual dummy variables  maybe not needed now??  GR
@@ -72,10 +79,12 @@ sampler.syn <- function(p, data, m, syn, visit.sequence,
         vname     <- dimnames(p$data)[[2]][j]
         
         if(print.flag & theMethod!="dummy"  & j<=ncol(data)) cat(" ",vname,sep="")
-        if(j%%10==0 & j<=ncol(data)) cat("\n    ")
+        if(j%%10==0 & j <= ncol(data)) cat("\n    ")
         
-        ya <-  1:nrow(p$data) 
-        ypa <- 1:k                  ############  new
+########*********
+        ya  <-  1:nrow(p$data) 
+        ypa <- 1:k    
+                     
         # ya=yavailable, ym=ymissing                                            
         if(any(p$rules[[j]]!="")) {
           com.rules  <- paste(p$rules[[j]],collapse=" | ")
@@ -85,48 +94,56 @@ sampler.syn <- function(p, data, m, syn, visit.sequence,
           evalrul.yp <- with(p$syn,eval(parse(text=com.rules)))         
           ypm        <- which(evalrul.yp==TRUE & !is.na(evalrul.yp))        
           ypa        <- setdiff(1:nrow(p$syn),ypm)       
-                    
         }                                                                       
            
+        # !="", !="dummy", !="passive"
         if(theMethod!="" & (!is.passive(theMethod)) & theMethod!="dummy" ){
-          if (theMethod %in% c("sample","sample.proper")) {
-            ##### new code for method sample
+          if (theMethod %in% c("sample","sample.proper","constant")) {
             y   <- p$data[ya, j]
             if (is.factor(y)) y <- y[,drop=TRUE]
             xp  <- length(ypa)
             x   <- length(ya)
             nam <- vname
             f   <- paste("syn", theMethod, sep = ".")
-            p$syn[ypa, j]  <- do.call(f, args = list(y=y,xp=xp,
+            synfun <- do.call(f, args = list(y=y,xp=xp,
               smoothing=p$smoothing[j],cont.na=p$cont.na[[j]],proper=proper, ...)) 
+            p$syn[ypa, j]  <- synfun$res
+            if (models) fits[[i]][[j]] <- synfun$fit
+            
           } else {
+          
             x    <- p$data[ya, p$predictor.matrix[j, ] == 1, drop = FALSE]
             xp   <- p$syn [ypa, p$predictor.matrix[j, ] == 1, drop = FALSE]
             y    <- p$data[ya, j]
             if (is.factor(y)) y <- y[,drop=TRUE]
             nam  <- vname
             f    <- paste("syn", theMethod, sep = ".")
-            keep <- remove.lindep.syn(x, y, ...)
-            x    <- x[, keep, drop = FALSE]
-            xp   <- xp[, keep, drop = FALSE]
-            #browser()
+            if(theMethod!="collinear"){                  #! BN to check
+              keep <- remove.lindep.syn(x, y, ...)
+              x    <- x[, keep, drop = FALSE]
+              xp   <- xp[, keep, drop = FALSE]
+            }                                            #! BN to check
             if (theMethod=="survctree") {
               if (p$event[j]==-1) yevent <- rep(1,length(y))                    #BN--27/05/2015 'p$' added
               else yevent  <- p$data[ya,p$event[j]]
               survres      <- do.call(f, args = c(list(y=y,yevent=yevent,
-                x=x,xp=xp,proper=proper),fun.args))
+                x=x,xp=xp,proper=proper), fun.args))
               p$syn[ypa, j] <- survres[[1]]# synthetic data survival goes to p$syn
               if (p$event[j]!=-1) p$syn[ypa,p$event[j]] <- survres[[2]] # synthetic data event goes to p$syn
-            }
-            else if (theMethod=="logreg" & p$denom[j]!=0) {     #GR denom new   #BN--27/05/2015 '& j<=ncol(data)' removed           
-              p$syn[ypa, j] <- do.call(f, args = list(y=y,x=x,xp=xp,
+              if (models) fits[[i]][[j]] <- survres$fit 
+            } else if (theMethod=="logreg" & p$denom[j]!=0) {     #GR denom new   #BN--27/05/2015 '& j<=ncol(data)' removed           
+              synfun <- do.call(f, args = list(y=y,x=x,xp=xp,
                 denom=p$data[ya,p$denom[j]],denomp=p$syn[ypa,p$denom[j]],       #BN--27/05/2015 'ya','ypa' added
                 proper=proper, ...))
-            }   
-            else {
-              p$syn[ypa, j] <- do.call(f, args = c(list(y=y,x=x,xp=xp,
-                smoothing=p$smoothing[j],proper=proper),fun.args))
+              p$syn[ypa, j] <- synfun$res
+              if (models) fits[[i]][[j]] <- synfun$fit           
+            } else {
+              synfun <- do.call(f, args = c(list(y=y, x=x, xp=xp,
+                smoothing=p$smoothing[j], proper=proper), fun.args))
+              p$syn[ypa, j] <- synfun$res
+              if (models) fits[[i]][[j]] <- synfun$fit
             }
+          
           }
 
           if(any(p$rules[[j]]!="")){
@@ -140,12 +157,15 @@ sampler.syn <- function(p, data, m, syn, visit.sequence,
               }
             }                 
           }  
-        }
+        } # end of !="", !="dummy", !="passive"
 
         else if (is.passive(theMethod)) {
           class0 <- class(p$syn[,j])
-          p$syn[,j] <- suppressWarnings(model.frame(as.formula(theMethod), p$syn, na.action=na.pass))	#BN 25/08 added suppressWarnings to avoid NAs by coersion for NAtemp
+          synfun <- syn.passive(data = p$syn, func = theMethod) 
+          #p$syn[,j] <- suppressWarnings(model.frame(as.formula(theMethod), p$syn, na.action=na.pass))	#BN 25/08 added suppressWarnings to avoid NAs by coersion for NAtemp
+          p$syn[, j] <- synfun$res
           class(p$syn[,j]) <- class0
+          if (models) fits[[i]][[j]] <- synfun$fit 
         }
 
         else if (theMethod=="dummy") {    # replace dummy variables in p$syn
@@ -158,26 +178,29 @@ sampler.syn <- function(p, data, m, syn, visit.sequence,
                   nrow=nrow(p$syn))
           p$syn[,j] <- as.numeric(p$syn[,j])
           remove("cat.columns")
+          if (models) fits[[i]][[j]] <- "dummy"    
         }
 
       } # end j loop 
     
       #if (k==dim(data)[1]) syn[[i]] <- p$syn[,1:dim(data)[2]]
       #else syn[[i]] <- p$syn[sample(1:dim(data)[1],k),1:dim(data)[2]]
+      
+########*********
       syn[[i]] <- p$syn[,1:dim(data)[2],drop=FALSE]
-      nms<-names(data)
+      nms <- names(data)
       # exclude unsynthesised if drop.pred.only set to true
       if (sum(pred.not.syn )>0) {
         syn[[i]] <- syn[[i]][,!pred.not.syn]
-        nms<-nms[!pred.not.syn]                               #GR save names to use below if data just one column
+        nms <- nms[!pred.not.syn]         # GR save names to use below if data just one column
       }
                                           # GR changes extra lines needed # to prevent a single character column being changed to a factor
-      chgetochar<- (sum(!pred.not.syn)==1 & class(syn[[i]][,1])=="character")       
+      chgetochar <- (sum(!pred.not.syn)==1 & class(syn[[i]][,1])=="character")       
   
       syn[[i]] <- as.data.frame(syn[[i]])
       if (chgetochar) {
-         syn[[i]][,1]<-as.character(syn[[i]][,1])
-         names(syn[[i]])<-nms
+         syn[[i]][,1] <- as.character(syn[[i]][,1])
+         names(syn[[i]]) <- nms
       }
    
 
@@ -189,29 +212,37 @@ sampler.syn <- function(p, data, m, syn, visit.sequence,
   #      }
   #    }
 
-      #turn NA level in factors to missing NA's
+      #turn NA level in factors / logical to missing NA's
       # and remove contrasts 
       for (j in (1:ncol(syn[[i]]))){
-        if(is.factor(syn[[i]][,j])) {
-        #  syn[[i]][,j] <- factor(syn[[i]][,j],exclude=NA,levels=levels(syn[[i]][,j]))
-          levels(syn[[i]][,j])[levels(syn[[i]][,j])=="NAtemp"] <- NA     #!BN 10/08/15 
-          attributes(syn[[i]][,j])$contrasts <- NULL
-        }
+        if(is.factor(syn[[i]][,j])) {                                    #!BN-20/04/16
+          if("NAlogical" %in% levels(syn[[i]][,j])){
+            levels(syn[[i]][,j])[levels(syn[[i]][,j])=="NAlogical"] <- NA
+            syn[[i]][,j] <- as.logical(syn[[i]][,j])
+          } else {                                                            
+        # syn[[i]][,j] <- factor(syn[[i]][,j],exclude=NA,levels=levels(syn[[i]][,j]))
+            levels(syn[[i]][,j])[levels(syn[[i]][,j])=="NAtemp"] <- NA     #!BN 10/08/15 
+          }
+        #! attributes(syn[[i]][,j])$contrasts <- NULL                           UNCOMMENT???? BN-28/04/16
+        }                                                                       
       }
 
-    } # end i loop
+    } # end i loop (m)
  } # end synthesising
   
  if (print.flag) cat("\n")
- return(syn)
+ return(list(syn = syn, fits = fits))
 }
 
 
+
+###-----remove.lindep.syn--------------------------------------------------
+
 remove.lindep.syn <- function(x, y, eps=0.00001, maxcor=0.99999, 
                               allow.na=FALSE, ...) {
   if (ncol(x)==0) return(NULL) 
   if (eps <= 0) stop("\n Argument 'eps' must be positive.")
-  xobs <- sapply(x,as.numeric)                                       #!BN1605 was xobs <- x
+  xobs <- sapply(x,as.numeric)                                       #!BN-1605 was xobs <- x
   yobs <- as.numeric(y)
   keep <- unlist(apply(xobs, 2, var) > eps)
   keep[is.na(keep)] <- FALSE
@@ -232,16 +263,3 @@ remove.lindep.syn <- function(x, y, eps=0.00001, maxcor=0.99999,
   #   " removed due to linear dependency",sep="")
   return(keep)
 }
-
-# make list of collinear variables
-find.collinear <- function(x, threshold=0.99999, ...) {
-  nvar      <- ncol(x)
-  x         <- data.matrix(x)
-  varnames  <- dimnames(x)[[2]]
-  z         <- suppressWarnings(cor(x, use="pairwise.complete.obs"))
-  hit       <- outer(1:nvar,1:nvar,"<") & (abs(z)>=threshold)
-  collvar   <- which (hit==1 ,arr.ind=TRUE)
-  collvar[] <- varnames[collvar]
-  collvarpairs <- paste(collvar[,1],collvar[,2],sep=" & ")
-  return(collvarpairs)
-}
